@@ -11,7 +11,7 @@ from benchmark_runner.common.oc.oc import OC
 from benchmark_runner.common.template_operations.template_operations import TemplateOperations
 from benchmark_runner.common.elasticsearch.elasticsearch_operations import ElasticSearchOperations
 from benchmark_runner.common.ssh.ssh import SSH
-from benchmark_runner.benchmark_operator.benchmark_operator_exceptions import ODFNonInstalled, SystemMetricsRequiredElasticSearch
+from benchmark_runner.benchmark_operator.benchmark_operator_exceptions import ODFNonInstalled
 from benchmark_runner.main.environment_variables import environment_variables
 from benchmark_runner.common.clouds.shared.s3.s3_operations import S3Operations
 from benchmark_runner.common.prometheus.prometheus_snapshot import PrometheusSnapshot
@@ -51,6 +51,8 @@ class BenchmarkOperatorWorkloadsOperations:
         self._es_user = self._environment_variables_dict.get('elasticsearch_user', '')
         self._es_password = self._environment_variables_dict.get('elasticsearch_password', '')
         self._timeout = int(self._environment_variables_dict.get('timeout', ''))
+        self._uuid = self._environment_variables_dict.get('uuid', '')
+        self._es_url_protocol = self._environment_variables_dict['elasticsearch_url_protocol']
         self._ssh = SSH()
         # Elasticsearch connection
         if self._es_host and self._es_port:
@@ -58,6 +60,7 @@ class BenchmarkOperatorWorkloadsOperations:
                                                            es_port=self._es_port,
                                                            es_user=self._es_user,
                                                            es_password=self._es_password,
+                                                           es_url_protocol=self._es_url_protocol,
                                                            timeout=self._timeout)
         # Generate template class
         self._template = TemplateOperations(workload=self._workload)
@@ -77,15 +80,6 @@ class BenchmarkOperatorWorkloadsOperations:
         self._oc.login()
         return self._oc
 
-    def __check_elasticsearch_exist_for_system_metrics(self):
-        """
-        This method check if elasticsearch exist for system metrics
-        :return:
-        """
-        if self._system_metrics == 'True':
-            if not self._elasticsearch:
-                raise SystemMetricsRequiredElasticSearch()
-
     @logger_time_stamp
     def update_node_selector(self, runner_path: str = environment_variables.environment_variables_dict['runner_path'], yaml_path: str = '', pin_node: str = ''):
         """
@@ -199,9 +193,9 @@ class BenchmarkOperatorWorkloadsOperations:
         # verify that data upload to elastic search
         if self._es_host:
             # workload: system metrics, in case of uperf take timestamp and not uperf_ts
-            self._verify_elasticsearch_data_uploaded(index=es_index, uuid=self._oc.get_long_uuid(workload=workload), workload='system-metrics', fast_check = True)
+            self._verify_elasticsearch_data_uploaded(index=es_index, uuid=self._oc.get_long_uuid(workload=workload), workload='system-metrics', fast_check=True)
 
-    def __get_metadata(self, kind: str = None, database: str = None, status: str = None, run_artifacts_url: str = None) -> dict:
+    def __get_metadata(self, kind: str = None, database: str = None, status: str = None, run_artifacts_url: str = None, uuid: str = None) -> dict:
         """
         This method return metadata kind and database argument are optional
         @param kind: optional: pod, vm, or kata
@@ -228,10 +222,11 @@ class BenchmarkOperatorWorkloadsOperations:
         if run_artifacts_url:
             metadata.update({'run_artifacts_url': run_artifacts_url})
         if database:
-            metadata.update({'vm_os_version': 'centos8'})
+            metadata.update({'vm_os_version': 'centos-stream8'})
         else:
             metadata.update({'vm_os_version': 'fedora34'})
-
+        if uuid:
+            metadata.update({'uuid': uuid})
         # for hammerdb
         if database == 'mssql':
             metadata.update({'db_version': 2019})
@@ -255,7 +250,7 @@ class BenchmarkOperatorWorkloadsOperations:
         """
         self.__es_operations.update_elasticsearch_index(index=index, id=id, metadata=self.__get_metadata(kind=kind, database=database, status=status, run_artifacts_url=run_artifacts_url))
 
-    def _upload_to_elasticsearch(self, index: str, kind: str, status: str, run_artifacts_url: str, database: str = ''):
+    def _upload_to_elasticsearch(self, index: str, kind: str, status: str, run_artifacts_url: str, database: str = '', uuid: str = ''):
         """
         This method upload to elasticsearch
         :param index:
@@ -265,21 +260,42 @@ class BenchmarkOperatorWorkloadsOperations:
         :param database:
         :return:
         """
-        self.__es_operations.upload_to_elasticsearch(index=index, data=self.__get_metadata(kind=kind, database=database, status=status, run_artifacts_url=run_artifacts_url))
+        self.__es_operations.upload_to_elasticsearch(index=index, data=self.__get_metadata(kind=kind, database=database, status=status, run_artifacts_url=run_artifacts_url, uuid=uuid))
 
-    def _verify_elasticsearch_data_uploaded(self, index: str, uuid: str, workload: str = '', fast_check: bool = False):
+    def _verify_elasticsearch_data_uploaded(self, index: str, uuid: str, workload: str = '', fast_check: bool = False, timeout: int = None):
         """
         This method verify that elasticsearch data uploaded
         :param index:
         :param uuid:
         :param workload:
         :param fast_check:
+        :param timeout:
         :return: ids
         """
         if workload:
-            return self.__es_operations.verify_elasticsearch_data_uploaded(index=index, uuid=uuid, workload=workload, fast_check=fast_check)
+            return self.__es_operations.verify_elasticsearch_data_uploaded(index=index, uuid=uuid, workload=workload, fast_check=fast_check, timeout=timeout)
         else:
-            return self.__es_operations.verify_elasticsearch_data_uploaded(index=index, uuid=uuid, workload=self._workload_name, fast_check=fast_check)
+            return self.__es_operations.verify_elasticsearch_data_uploaded(index=index, uuid=uuid, workload=self._workload_name, fast_check=fast_check, timeout=timeout)
+
+    def _upload_workload_to_elasticsearch(self, index: str, kind: str, status: str, result: dict = None):
+        """
+        This method upload to elasticsearch
+        :param index:
+        :param kind:
+        :param status:
+        :param result:
+        :return:
+        """
+        self.__es_operations.upload_to_elasticsearch(index=index, data=self.__get_metadata(kind=kind, status=status, result=result))
+
+    def _verify_elasticsearch_workload_data_uploaded(self, index: str, uuid: str):
+        """
+        This method verify that elasticsearch data uploaded
+        :param index:
+        :param uuid:
+        :return:
+        """
+        self.__es_operations.verify_elasticsearch_data_uploaded(index=index, uuid=uuid)
 
     def _create_vm_log(self, labels: list) -> str:
         """
@@ -293,7 +309,7 @@ class BenchmarkOperatorWorkloadsOperations:
             self._oc.save_vm_log(vm_name=vm_name)
         return vm_name
 
-    def __create_pod_log(self, label: str = '', database: str = '') -> str:
+    def _create_pod_log(self, label: str = '', database: str = '') -> str:
         """
         This method create pod log per workload
         :param label:pod label
@@ -306,7 +322,7 @@ class BenchmarkOperatorWorkloadsOperations:
         else:
             self._oc.save_pod_log(pod_name=pod_name)
 
-    def __get_run_artifacts_hierarchy(self, workload_name: str = '', is_file: bool = False):
+    def _get_run_artifacts_hierarchy(self, workload_name: str = '', is_file: bool = False):
         """
         This method return log hierarchy
         :param workload_name: workload name
@@ -333,21 +349,21 @@ class BenchmarkOperatorWorkloadsOperations:
         :param labels: list of labels of pod names - using it when it different from workload name
         :return: run artifacts url
         """
-        self.__create_pod_log(label='benchmark-controller-manager')
+        self._create_pod_log(label='benchmark-controller-manager')
         if environment_variables.environment_variables_dict['system_metrics'] == 'True':
-            self.__create_pod_log(label='system-metrics')
+            self._create_pod_log(label='system-metrics')
         # for vm call to create_vm_log
         if pod:
             # workload that contains 2 pods
             if labels:
                 for label in labels:
-                    self.__create_pod_log(label=label)
+                    self._create_pod_log(label=label)
             else:
-                self.__create_pod_log(label=workload)
+                self._create_pod_log(label=workload)
             if database:
-                self.__create_pod_log(database=database)
+                self._create_pod_log(database=database)
         workload_name = self._workload_name
-        return os.path.join(self._environment_variables_dict.get('run_artifacts_url', ''), f'{self.__get_run_artifacts_hierarchy(workload_name=workload_name, is_file=True)}-{self._time_stamp_format}.tar.gz')
+        return os.path.join(self._environment_variables_dict.get('run_artifacts_url', ''), f'{self._get_run_artifacts_hierarchy(workload_name=workload_name, is_file=True)}-{self._time_stamp_format}.tar.gz')
 
     def __make_run_artifacts_tarfile(self, workload: str):
         """
@@ -368,16 +384,15 @@ class BenchmarkOperatorWorkloadsOperations:
         """
         workload = self._workload_name
         tar_run_artifacts_path = self.__make_run_artifacts_tarfile(workload)
-        run_artifacts_hierarchy = self.__get_run_artifacts_hierarchy(workload_name=workload)
+        run_artifacts_hierarchy = self._get_run_artifacts_hierarchy(workload_name=workload)
         # Upload when endpoint_url is not None
-        if self._endpoint_url:
-            s3operations = S3Operations()
-            # change workload to key convention
-            upload_file = f"{workload}-{self._time_stamp_format}.tar.gz"
-            s3operations.upload_file(file_name_path=tar_run_artifacts_path,
-                                     bucket=self._environment_variables_dict.get('bucket', ''),
-                                     key=run_artifacts_hierarchy,
-                                     upload_file=upload_file)
+        s3operations = S3Operations()
+        # change workload to key convention
+        upload_file = f"{workload}-{self._time_stamp_format}.tar.gz"
+        s3operations.upload_file(file_name_path=tar_run_artifacts_path,
+                                 bucket=self._environment_variables_dict.get('bucket', ''),
+                                 key=run_artifacts_hierarchy,
+                                 upload_file=upload_file)
         # remove local run artifacts workload folder
         # verify that its not empty path
         if len(self._run_artifacts_path) > 3 and self._run_artifacts_path != '/' and self._run_artifacts_path and tar_run_artifacts_path and os.path.isfile(tar_run_artifacts_path) and not self._save_artifacts_local:
@@ -421,7 +436,7 @@ class BenchmarkOperatorWorkloadsOperations:
         :return:
         """
         workload_name = self._workload.split('_')
-        if self._odf_pvc == 'True' and workload_name[0] in self._workloads_odf_pvc:
+        if workload_name[0] in self._workloads_odf_pvc:
             if not self._oc.is_odf_installed():
                 raise ODFNonInstalled()
 
@@ -434,7 +449,14 @@ class BenchmarkOperatorWorkloadsOperations:
         # make undeploy benchmark controller manager if exist
         self.make_undeploy_benchmark_controller_manager_if_exist(runner_path=environment_variables.environment_variables_dict['runner_path'])
         if 'hammerdb' in self._workload:
-            self._oc.delete_all_resources(resources=['deployments', 'services', 'pvc'], namespace=f"{self._workload.split('_')[2]}-db")
+            self._oc.delete_namespace(namespace=f"{self._workload.split('_')[2]}-db")
+
+    @logger_time_stamp
+    def clear_nodes_cache(self):
+        """
+        This method clear nodes cache
+        """
+        self._oc.clear_node_caches()
 
     def initialize_workload(self):
         """
@@ -442,19 +464,22 @@ class BenchmarkOperatorWorkloadsOperations:
         :return:
         """
         self.delete_all()
-        # check that there is elasticsearch when system metric is True
-        self.__check_elasticsearch_exist_for_system_metrics()
+        self.clear_nodes_cache()
         # make deploy benchmark controller manager
         self.make_deploy_benchmark_controller_manager(runner_path=environment_variables.environment_variables_dict['runner_path'])
-        self.odf_pvc_verification()
+        if self._odf_pvc == 'True':
+            self.odf_pvc_verification()
         self._template.generate_yamls()
-        self.start_prometheus()
+        if self._enable_prometheus_snapshot:
+            self.start_prometheus()
 
     def finalize_workload(self):
         """
         This method includes all the finalization of workload
         :return:
         """
-        self.end_prometheus()
-        self.upload_run_artifacts_to_s3()
+        if self._enable_prometheus_snapshot:
+            self.end_prometheus()
+        if self._endpoint_url:
+            self.upload_run_artifacts_to_s3()
         self.delete_all()
