@@ -1,4 +1,5 @@
 
+import ast
 import os
 import datetime
 import tarfile
@@ -18,6 +19,7 @@ from benchmark_runner.common.clouds.IBM.ibm_operations import IBMOperations
 
 
 class WorkloadsOperations:
+    oc = None
     """
     This class run workloads
     """
@@ -46,6 +48,14 @@ class WorkloadsOperations:
         self._es_port = self._environment_variables_dict.get('elasticsearch_port', '')
         self._es_user = self._environment_variables_dict.get('elasticsearch_user', '')
         self._es_password = self._environment_variables_dict.get('elasticsearch_password', '')
+        self._es_url_protocol = self._environment_variables_dict['elasticsearch_url_protocol']
+        self._scale = self._environment_variables_dict.get('scale', '')
+        if self._scale:
+            self._scale = int(self._scale)
+            self._scale_nodes = self._environment_variables_dict.get('scale_nodes', '')
+            self._scale_node_list = ast.literal_eval(self._scale_nodes)
+        else:
+            self._scale_node_list = []
         self._timeout = int(self._environment_variables_dict.get('timeout', ''))
         # Elasticsearch connection
         if self._es_host and self._es_port:
@@ -53,15 +63,30 @@ class WorkloadsOperations:
                                                            es_port=self._es_port,
                                                            es_user=self._es_user,
                                                            es_password=self._es_password,
+                                                           es_url_protocol=self._es_url_protocol,
                                                            timeout=self._timeout)
         # Generate templates class
         self._template = TemplateOperations(workload=self._workload)
         # set oc login
-        self._oc = self.set_login(kubeadmin_password=self._kubeadmin_password)
+
+        if WorkloadsOperations.oc is None:
+            WorkloadsOperations.oc = self.set_login(kubeadmin_password=self._kubeadmin_password)
+        self._oc = WorkloadsOperations.oc
+
         # PrometheusSnapshot
         if self._enable_prometheus_snapshot:
             self._snapshot = PrometheusSnapshot(oc=self._oc, artifacts_path=self._run_artifacts_path, verbose=True)
 
+    def __get_workload_file_name(self, workload):
+            """
+            This method returns workload name
+            :return:
+            """
+            if self._scale:
+                return f'{workload}-scale-{self._time_stamp_format}'
+            else:
+                return f'{workload}-{self._time_stamp_format}'
+
     def set_login(self, kubeadmin_password: str = ''):
         """
         This method set oc login
@@ -78,13 +103,13 @@ class WorkloadsOperations:
         This method delete all resources in namespace
         :return:
         """
-        self._oc.delete_all_resources(resources=['vm', 'pods', 'pvc'])
-    
+        self._oc.delete_namespace()
+
     @logger_time_stamp
     def start_prometheus(self):
         """
         This method start collection of Prometheus snapshot
-        :return: 
+        :return:
         """
         if self._enable_prometheus_snapshot:
             try:
@@ -93,12 +118,12 @@ class WorkloadsOperations:
                 raise PrometheusSnapshotError(err)
             except Exception as err:
                 raise err
-    
+
     @logger_time_stamp
     def end_prometheus(self):
         """
         This method retrieve the Prometheus snapshot
-        :return: 
+        :return:
         """
         if self._enable_prometheus_snapshot:
             try:
@@ -107,7 +132,7 @@ class WorkloadsOperations:
                 raise PrometheusSnapshotError(err)
             except Exception as err:
                 raise err
-            
+
     @logger_time_stamp
     def odf_pvc_verification(self):
         """
@@ -115,7 +140,7 @@ class WorkloadsOperations:
         :return:
         """
         workload_name = self._workload.split('_')
-        if self._odf_pvc == 'True' and workload_name[0] in self._workloads_odf_pvc:
+        if workload_name[0] in self._workloads_odf_pvc:
             if not self._oc.is_odf_installed():
                 raise ODFNonInstalled()
 
@@ -131,7 +156,7 @@ class WorkloadsOperations:
             self._oc.save_vm_log(vm_name=vm_name)
         return vm_name
 
-    def __create_pod_log(self, pod: str = ''):
+    def _create_pod_log(self, pod: str = ''):
         """
         This method create pod log per workload
         :param pod: pod name
@@ -140,7 +165,7 @@ class WorkloadsOperations:
         pod_name = self._oc.get_pod(label=pod)
         return self._oc.save_pod_log(pod_name=pod_name)
 
-    def __get_run_artifacts_hierarchy(self, workload_name: str = '', is_file: bool = False):
+    def _get_run_artifacts_hierarchy(self, workload_name: str = '', is_file: bool = False):
         """
         This method return log hierarchy
         :param workload_name: workload name
@@ -171,6 +196,14 @@ class WorkloadsOperations:
         except ValueError:
             return False
 
+    def _create_scale_logs(self):
+        """
+        The method create scale logs
+        :return:
+        """
+        self._create_pod_log(pod='state-signals-exporter')
+        self._create_pod_log(pod='redis-master')
+
     def _create_pod_run_artifacts(self, pod_name: str):
         """
         This method create pod run artifacts
@@ -178,7 +211,7 @@ class WorkloadsOperations:
         :return: run results dict
         """
         result_list = []
-        pod_log_file = self.__create_pod_log(pod=pod_name)
+        pod_log_file = self._create_pod_log(pod=pod_name)
         workload_name = self._environment_variables_dict.get('workload', '').replace('_', '-')
         # csv to dictionary
         the_reader = DictReader(open(pod_log_file, 'r'))
@@ -190,7 +223,8 @@ class WorkloadsOperations:
                 elif value == 'n/a':
                     line_dict[key] = 0.0
             line_dict['pod_name'] = pod_name
-            line_dict['run_artifacts_url'] = os.path.join(self._run_artifacts_url, f'{self.__get_run_artifacts_hierarchy(workload_name=workload_name, is_file=True)}-{self._time_stamp_format}.tar.gz')
+            workload = self.__get_workload_file_name(workload=self._get_run_artifacts_hierarchy(workload_name=workload_name, is_file=True))
+            line_dict['run_artifacts_url'] = os.path.join(self._run_artifacts_url, f'{workload}.tar.gz')
             result_list.append(dict(line_dict))
         return result_list
 
@@ -205,11 +239,16 @@ class WorkloadsOperations:
         result_list = []
         results_list = self._oc.extract_vm_results(vm_name=vm_name, start_stamp=start_stamp, end_stamp=end_stamp)
         workload_name = self._environment_variables_dict.get('workload', '').replace('_', '-')
+        # save scale pod logs
+        if self._scale:
+            self._create_pod_log(pod='state-signals-exporter')
+            self._create_pod_log(pod='redis-master')
         # insert results to csv
         csv_result_file = os.path.join(self._run_artifacts_path, 'vdbench_vm_result.csv')
         with open(csv_result_file, 'w') as out:
             for row in results_list:
-                out.write(f'{row[0].strip()}\n')
+                if row:
+                    out.write(f'{row[0].strip()}\n')
         # csv to dictionary
         the_reader = DictReader(open(csv_result_file, 'r'))
         for line_dict in the_reader:
@@ -220,7 +259,8 @@ class WorkloadsOperations:
                 elif value == 'n/a':
                     line_dict[key] = 0.0
             line_dict['vm_name'] = vm_name
-            line_dict['run_artifacts_url'] = os.path.join(self._run_artifacts_url, f'{self.__get_run_artifacts_hierarchy(workload_name=workload_name, is_file=True)}-{self._time_stamp_format}.tar.gz')
+            workload = self.__get_workload_file_name(workload=self._get_run_artifacts_hierarchy(workload_name=workload_name, is_file=True))
+            line_dict['run_artifacts_url'] = os.path.join(self._run_artifacts_url, f'{workload}.tar.gz')
             result_list.append(dict(line_dict))
         return result_list
 
@@ -231,28 +271,28 @@ class WorkloadsOperations:
         """
         tar_run_artifacts_path = f"{self._run_artifacts_path}.tar.gz"
         with tarfile.open(tar_run_artifacts_path, mode='w:gz') as archive:
-            archive.add(self._run_artifacts_path, arcname=f'{workload}-{self._time_stamp_format}', recursive=True)
+            workload_file_name = self.__get_workload_file_name(workload)
+            archive.add(self._run_artifacts_path, arcname=workload_file_name, recursive=True)
         return tar_run_artifacts_path
 
     @logger_time_stamp
     def upload_run_artifacts_to_s3(self):
         """
         This method uploads log to s3
-        :param workload:
         :return:
         """
         workload = self._workload.replace('_', '-')
         tar_run_artifacts_path = self.__make_run_artifacts_tarfile(workload)
-        run_artifacts_hierarchy = self.__get_run_artifacts_hierarchy(workload_name=workload)
+        run_artifacts_hierarchy = self._get_run_artifacts_hierarchy(workload_name=workload)
         # Upload when endpoint_url is not None
-        if self._endpoint_url:
-            s3operations = S3Operations()
-            # change workload to key convention
-            upload_file = f"{workload}-{self._time_stamp_format}.tar.gz"
-            s3operations.upload_file(file_name_path=tar_run_artifacts_path,
-                                     bucket=self._environment_variables_dict.get('bucket', ''),
-                                     key=run_artifacts_hierarchy,
-                                     upload_file=upload_file)
+        s3operations = S3Operations()
+        # change workload to key convention
+        workload_file_name = self.__get_workload_file_name(workload)
+        upload_file = f"{workload_file_name}.tar.gz"
+        s3operations.upload_file(file_name_path=tar_run_artifacts_path,
+                                 bucket=self._environment_variables_dict.get('bucket', ''),
+                                 key=run_artifacts_hierarchy,
+                                 upload_file=upload_file)
         # remove local run artifacts workload folder
         # verify that its not empty path
         if len(self._run_artifacts_path) > 3 and self._run_artifacts_path != '/' and self._run_artifacts_path and tar_run_artifacts_path and os.path.isfile(tar_run_artifacts_path) and not self._save_artifacts_local:
@@ -276,7 +316,7 @@ class WorkloadsOperations:
                     'odf_version': self._oc.get_odf_version(),
                     'runner_version': self._build_version,
                     'version': int(self._build_version.split('.')[-1]),
-                    'vm_os_version': 'centos8',
+                    'vm_os_version': 'centos-stream8',
                     'ci_date': datetime.datetime.now().strftime(date_format),
                     'uuid': self._uuid,
                     'pin_node1': self._pin_node1,
@@ -285,6 +325,13 @@ class WorkloadsOperations:
             metadata.update({'kind': kind})
         if status:
             metadata.update({'run_status': status})
+        if self._scale:
+            metadata.update({'scale': int(self._scale)})
+            count = 0
+            for scale_node in range(len(self._scale_node_list)):
+                for scale_num in range(self._scale):
+                    count += 1
+                    metadata.update({f'{kind}-scale-node-{count}': self._scale_node_list[scale_node]})
         if result:
             metadata.update(result)
 
@@ -336,21 +383,33 @@ class WorkloadsOperations:
         metadata.update({'status': status, 'status#': status_dict[status], 'ci_minutes_time': ci_minutes_time, 'benchmark_operator_id': benchmark_operator_id, 'benchmark_wrapper_id': benchmark_wrapper_id, 'ocp_install_minutes_time': ocp_install_minutes_time, 'ocp_resource_install_minutes_time': ocp_resource_install_minutes_time})
         self.__es_operations.upload_to_elasticsearch(index=es_index, data=metadata)
 
+    @logger_time_stamp
+    def clear_nodes_cache(self):
+        """
+        This method clear nodes cache
+        """
+        self._oc.clear_node_caches()
+
     def initialize_workload(self):
         """
-        This method includes all the initialization of workload 
-        :return: 
+        This method includes all the initialization of workload
+        :return:
         """
         self.delete_all()
-        self.odf_pvc_verification()
-        self._template.generate_yamls()
-        self.start_prometheus()
-        
+        self.clear_nodes_cache()
+        if self._odf_pvc == 'True':
+            self.odf_pvc_verification()
+        self._template.generate_yamls(scale=self._scale, scale_nodes=self._scale_node_list)
+        if self._enable_prometheus_snapshot:
+            self.start_prometheus()
+
     def finalize_workload(self):
         """
-        This method includes all the finalization of workload 
-        :return: 
+        This method includes all the finalization of workload
+        :return:
         """
-        self.end_prometheus()
-        self.upload_run_artifacts_to_s3()
+        if self._enable_prometheus_snapshot:
+            self.end_prometheus()
+        if self._endpoint_url:
+            self.upload_run_artifacts_to_s3()
         self.delete_all()
